function [fhat, se, mse, err, im_orig, im_noisy, im_recovered] = est_relevances_denoising_im(im, q, sigma, structure_learning, a, beta, seed)
% Given an image, split it into a set of qxq images, add Gaussian noise to each of them, 
% then denoise using wavelets and the (structured) sparsity assumption.
%
% Input: im -- a matrix: the greyscale image that we will first add noise to, then denoise
%        q -- a scalar in {4,8,16,32,64}: we extract small images of size q x q from im
%        sigma -- a scalar: the square root of the variance of the Gaussian noise    
%        structure_learning -- a scalar in {1,2,3}, specifying the groups for which
%                              the weights are being learned: 
%                              1 -- LASSO-like: only singletons with the same relevance; 
%                              2 -- weighted LASSO-like: only singletons with individual  
%                                   relevances;
%                              3 -- structured: all ascending trees in the wavelet tree.
%        a -- a scalar: the shape parameter of Student's t, used for inference       
%        beta -- a scalar: the regularization parameter
%        seed -- a scalar: the random seed used for data generation
% Output: fhat -- an NG x 1 vector: an estimate of the relevance for each group       
%         se -- a 1 x K vector: squared difference (error) between the predicted and the
%               real signals (small images)
%         mse_te -- mean squared error
%         err_te -- 95%-confidence error bar around the mean
%         im_orig -- a matrix: the original image (the same as im)
%         im_noisy -- a matrix: the image with added noise
%         im_recovered -- a matrix: the image reconstructed from a non-overlapping subset
%                         of predicted small images
%
% Copyright 2014 Nino Shervashidze

if nargin < 7 seed = 1; end
randn('state',seed);
rand('state',seed);

% fixed parameters
numiter = 2000;
load(['bases/basis_',num2str(q),'x',num2str(q),'.mat']);
objhowoften = 10;

%% DATA PREPARATION
% we take qxq images from the input image, therefore P=qxq
P = q*q;
% m will ensure that we have at least approximately 1000 qxq images
m = max(1, sqrt(1024/(numel(im)/P)));

K = 1024;
% generate data using a pre-computed wavelet basis
w = zeros(P,K);
k = 1;
imagette_means = zeros(1,K);
for i=1:ceil(q/m):size(im,1)-q+1
    for j=1:ceil(q/m):size(im,2)-q+1
        imagette=im(i:i+q-1,j:j+q-1);
        imagette_means(k) = mean(imagette(:));
        imagette = imagette - imagette_means(k);
        w(:,k) = basis*imagette(:);
        k=k+1;
    end
end
K=k-1;
w = w(:,1:K);
imagette_means = imagette_means(1:K);
% add noise
y = w + sigma*randn(P,K);

%% INFERENCE
addpath ../variational/
switch structure_learning
  case 1,
    % LASSO-like
    groups = eye(P);
    [fhat,what] = variational_lasso(y, sigma, groups, a, beta, numiter, objhowoften); 
  case 2,
    % weighted LASSO-like
    groups = eye(P);    
    [fhat,what] = variational(y, sigma, groups, a, beta, numiter, objhowoften); 
  case 3,
    % including non-singletons
    groups = get_groups_haar(P,2);
    %[fhat,what] = variational_OLD(y, sigma, groups, a, beta, numiter, 0.001); 
    [fhat,what] = variational(y, sigma, groups, a, beta, numiter, objhowoften); 
end
se  = sum((w-what).^2);
mse = mean(se);
err = 1.96*std(se)/sqrt(K);

%% IMAGE RECONSTRUCTION
% Reconstruct images using y and what (this is just to be able to visualize
% the results for debugging reasons more than anything else -- in fact, if  
% the small images have overlaps, then not all of them will be used here).
im_noisy = zeros(size(im));
im_recovered = zeros(size(im));
im_orig = zeros(size(im));
k = 1;
for i=1:q:size(im,1)
    for j=1:q:size(im,2)
        if k<=K 
            if rem(i,q)==1 && rem(j,q)==1
                im_orig(i:i+q-1,j:j+q-1)=reshape(basis'*w(:,k),[q q])+imagette_means(k);
                im_noisy(i:i+q-1,j:j+q-1)=reshape(basis'*y(:,k),[q q])+imagette_means(k);
                im_recovered(i:i+q-1,j:j+q-1)=reshape(basis'*what(:,k),[q q])+imagette_means(k);
            end
            k = k+1;
        else
            im_orig(i:i+q-1,j:j+q-1)=0;
            im_noisy(i:i+q-1,j:j+q-1)=0;
            im_recovered(i:i+q-1,j:j+q-1)=0;
        end
    end
end

end


