function [groups, parent] = get_groups_haar(P, type)
% Given the dimension of the signal in the wavelet basis, construct
% groups of type 1 (default) - all rooted paths in the wavelet tree, 
% or 2 - all descending paths in the wavelet tree.
%
% Input: P -- a scalar: the dimension of the signal
%        type -- a scalar in {1,2}: 1  (default) - all rooted paths, 2 - all
%                descending paths.
% Output: groups -- an NG x P binary matrix of the considered groups: for i=1...NG,
%                   the i-th row encodes which of the P variables make up the
%                   i-th group
%         parent -- a column vector of size P, encoding the tree structure. parent(i) 
%                   indicates the id of the parent of the node i. If i is the root,
%                   parent(i)=0.
%                     
% The ordering of nodes here corresponds to the ordering used in the toolbox
% matlabPyrTools (by E. Simoncelli), turned upside down.
    
if nargin < 2 type = 1; end

% Generally, ntrees=3, vertical/horizontal/diagonal (more for overcomplete basis)
ntrees           = 3;
nnodes           = P;

% Find the parent of each node 
parent = zeros(P,1);
parent(1) = 0;
if P > 1
    parent(2:4) = 1;
end
p_first = 1 + 1; c_first = 1 + ntrees + 1;
for level = 1:log2(sqrt(P))-1
    for i = 1:ntrees*(2^(level-1))^2
        parent(c_first+4*(i-1) : c_first+4*i-1) = p_first + i - 1;
    end
    c_first = c_first+4*ntrees*(2^(level-1))^2;
    p_first = p_first + ntrees*(2^(level-1))^2;
end

% Make groups
switch type
  case 1,
    % Groups are singletons and all rooted paths
    groups = zeros(P-1,P);
    for i = 2:P   
        j=i;
        while j>1
            groups(i-1,j)=1;
            j=parent(j);
        end
    end
    groups(:,1)=1;
    groups=[eye(P); groups];
  case 2,
    % Groups are all ascending paths
    np = 1;
    depth = 1;
    for i = 2:log2(sqrt(P))
        np = np+3*i*4^(i-1);
        % i is depth
    end
    groups = zeros(np,P);
    k=1;
    for i = 1:P
        j=i;
        groups(k,j)=1;
        k=k+1;
        j=parent(j);
        while j>=1
            groups(k,:)=groups(k-1,:);
            groups(k,j)=1;
            k=k+1;
            j=parent(j);
        end
    end
end
end
