function [fhat_te, se_te_per_task, mse_te, err_te, K, beta_star, fhat_tr, se_tr_per_task] = est_relevances_denoising(P, a, seed, structure_data, structure_learning, sigma)
% Generate data from the model, estimate group relevances and the signal using different values for the
% regularization hyperparameter beta, choose the value of beta minimizing the denoising error, and use
% it on held out data.
% 
% Input: P -- a scalar: the dimension of each signal w^k to be generated
%        a -- a scalar: the shape parameter of Student's t, used for data generation and
%             inference
%        seed -- a scalar: the random seed used in data generation
%        structure_data -- a scalar in {1,2,3}, the data generation model: 
%                          1 -- the relevant groups are singletons {1},{2},...,{ceil(P/2)}
%                          2 -- the relevant group is {1,2,...,ceil(P/2)}
%                          3 -- the relevant groups are {1},{1,2},...,{1,2,...,ceil(P/2)}
%        structure_learning -- a scalar in {1,2,3}, specifying the groups for which
%                              the weights are being learned: 
%                              1 -- LASSO-like: only singletons with the same relevance; 
%                              2 -- weighted LASSO-like: only singletons with individual  
%                                   relevances;
%                              3 -- structured: all groups of the form {1,...,Q} with Q 
%                                   in {2,...,P}, and all singletons.
%        sigma -- a scalar: the level of the Gaussian noise, used in data
%                 generation
% Output: fhat_te -- an NG x 1 vector: an estimate of the relevance for each group on the
%                    test data
%         se_tr_per_task -- a 1 x K vector: the squared error on each of the tasks in the
%                           test set
%         mse_te -- mean squared error on the test data
%         err_te -- 95%-confidence error bar around the mean on the test data
%         K -- a scalar: the number of tasks
%         beta_star -- a scalar: the value of the regularization parameter achieving the
%                      lowest square error averaged over the tasks
%         fhat_tr -- a cell array of NG x 1 vectors: the estimated value of f for each of 
%                    the considered values of the regularization parameter on the training 
%                    data
%         se_tr_per_task -- a cell array of 1 x K vectors: each representing the squared
%                           error on each of the tasks in the training set for each of the
%                           considered values for the regularization parameter 
%
% Copyright 2014 Nino Shervashidze    

randn('state',seed);
rand('state',seed);

% parameters
K = 1000;
fon = .1;
foff = 100;
if a > 1 fon = fon/(a-1); foff = foff/(a-1); end
betarange = [0 0.0001 0.001 0.01 0.02 0.05 0.1];


% parameters specifically for the variational
numiter = 1000;
objhowoften = 10;

% define groups: singletons and consecutive chunks starting at 1
groups = [eye(P); tril(ones(P-1,P),1)];
NG = size(groups,1);

% depending on structure, design f
Q = ceil(P/2);
switch structure_data
  case 1, 
    % singletons {1},...,{Q}
    f = foff*ones(NG,1);
    f(1:Q) = fon;
  case 2,
    % group {1,...,Q}
    f = foff*ones(NG,1);
    f(P+Q-1) = fon;
  case 3,
    % all groups {1,...,q} q=1,...,Q.
    f = foff*ones(NG,1);
    f([1,P+1:P+Q-1]) = fon;
    sigma = sqrt(3)*sigma;
    % so that the signal-to-noise ratio is
    % the same here and in previous cases
end


% generate data
w = zeros(P,K);
y = zeros(P,K);

for k=1:K
    for g=1:NG
        w(:,k) = w(:,k) + mvnrnd(zeros(P,1), 1./gamrnd(a,f(g)) * diag(groups(g,:)))';
    end
end

% add noise
y = w + sigma*randn(P,K);


addpath ../variational/
% K/nf is the test set size
nf = 2;
fs = ceil(K/nf); K_tr = K-fs;

% training and test data
w_tr = w(:,[fs+1:K]);
w_te = w(:,[1:fs]);
y_tr = y(:,[fs+1:K]);
y_te = y(:,[1:fs]);

% find the optimal beta on the training data
mse_tr = zeros(size(betarange));
fhat_tr = cell(size(betarange));
se_tr_per_task = cell(size(betarange));
for b = 1:length(betarange)
    beta = betarange(b);
    switch structure_learning
      case 1,
        % lasso-type
        groups = eye(P);
        [fhat_tr{b},what_tr,bound] = variational_lasso(y_tr, sigma, groups, a, beta, numiter, objhowoften);
      case 2,
        % weighted lasso-type
        groups = eye(P);
        [fhat_tr{b},what_tr,bound] = variational(y_tr, sigma, groups, a, beta, numiter, objhowoften);
      case 3,
        % including non-singletons
        groups = [eye(P); tril(ones(P-1,P),1)];
        [fhat_tr{b},what_tr,bound] = variational(y_tr, sigma, groups, a, beta, numiter, objhowoften);
    end
    se_tr_per_task{b} = sum((w_tr-what_tr).^2);    
    mse_tr(b) = sum(sum(se_tr_per_task{b},2)/K_tr);
end 
b_star = find(mse_tr==min(mse_tr))
beta_star = betarange(b_star(1));

% test the optimal beta on test data
switch structure_learning
  case 1,
    % lasso-type
    groups = eye(P);
    [fhat_te,what_te,bound] = variational_lasso(y_te, sigma, groups, a, beta_star, numiter, objhowoften);
  case 2,
    % weighted lasso-type
    groups = eye(P);
    [fhat_te,what_te,bound] = variational(y_te, sigma, groups, a, beta_star, numiter, objhowoften);
  case 3,
    % including non-singletons
    groups = [eye(P); tril(ones(P-1,P),1)];
    [fhat_te,what_te,bound] = variational(y_te, sigma, groups, a, beta_star, numiter, objhowoften);
end
% test mse in this training-test split
se_te_per_task = sum((w_te-what_te).^2);

mse_te = mean(se_te_per_task);
err_te = 1.96*std(se_te_per_task)/sqrt(K-K_tr);

end
