function [fhat_star, sigmahat_star] = est_scale_1d(a, seed, method, f, sigma, beta, sigmaknown)
% Estimate the inverse scale parameter of the 1-dimensional Student's t,
% corrupted with Gaussian noise, based on data from the model.
% 
% Input: a -- a scalar: the shape parameter of Student's t, used in data 
%             generation and inference
%        seed -- a scalar: the random seed used in data generation
%        method -- a scalar in {1,2}: 1 -- approximate maximum likelihood
%                  by numerically solving the intractable integral in the
%                  likelihood expression, 2 -- variational method
%        f -- a scalar: the inverse scale parameter of Student's t, used in
%             data generation, which we are trying to recover from data
%        sigma -- a scalar: the level of the Gaussian noise, used in data
%                 generation
%        beta -- a scalar (positive, small): the regularization parameter
%        sigmaknown -- a boolean (default value 1), specifying whether the
%                      value of sigma is supposed known (sigmaknown=1) or
%                      also has to be inferred
% Output: fhat_star -- a scalar: the estimated value of f
%         sigmahat_star (if sigmaknown=0) -- a scalar: the estimated value
%                                            of sigma
%
% Copyright 2014 Nino Shervashidze    

if nargin < 7 sigmaknown = 1; end
sigmahat_star = 0;

randn('state',seed);
rand('state',seed);

% fixed parameters
K = 10000;
tol = .0001;
frange = [1/100, 1/25,1/10,1/4, 1/2.8, 1/2, 1/1.4, 1, 1.4,  2, 2.8, 4,10, 25,10^5];
if a > 1 frange = frange/(a-1); end

if ~sigmaknown
    sigmarange = 10.^[-2:1:2];
else
    sigmarange = sigma;
end

% generate data
for k=1:K
    w(k) = mvnrnd(0,1/gamrnd(a,f)); 
end

% add noise
for k=1:K
    y(k) = w(k) + sigma*randn;
end

% choose method
switch method
  case 1, % Grid search-based minimization of minus log likelihood
    Obj = [];
    obj_star = Inf;
    for sigmahat = sigmarange
        for fhat = frange
            obj = 0;
            for k=1:K
                obj = obj - log(num_int_GaussStudent(y(k), sigmahat, a, fhat, beta));
            end
            num2str([fhat sigmahat obj])
            if obj < obj_star
                fhat_star = fhat;
                sigmahat_star = sigmahat;
                obj_star = obj;
            end
            Obj = [Obj,obj];
        end
    end
  case 2, % Variational
    if ~sigmaknown
        [fhat_star,sigmasqhat_star,zeta] = variational_sigmaunknown(y, a, tol,beta);
        sigmahat_star = sqrt(sigmasqhat_star);
    else
        [fhat_star,zeta] = variational(y, sigma, a, tol,beta); 
    end
end

end

function [S,gs,wrange] = num_int_GaussStudent(y, sigma, a, f, beta)
bounds = [y-10*sigma/f^.1,y+10*sigma/f^.1];
interval =  (bounds(2)-bounds(1))/1000;
wrange = bounds(1):interval:bounds(2);

S = 0;
gs_left = GS(wrange(1), y, sigma, a, f, beta);
gs=[gs_left];
for i = wrange
    w_left=i; w_right=i+interval;
    gs_right = GS(w_right, y, sigma, a, f, beta);
    S = S + interval/2 * (gs_left + gs_right);
    gs_left = gs_right;
    gs = [gs, gs_right];
end
threshold = (max(gs)-min(gs))/1000;
bounds(1) = wrange(max(min(find(gs > threshold))-1,1));
bounds(2) = wrange(min(max(find(gs > threshold))+1,1001));
interval =  (bounds(2)-bounds(1))/1000;
wrange = bounds(1):interval:bounds(2);

S = 0;
gs_left = GS(wrange(1), y, sigma, a, f, beta);
gs=[gs_left];
for i = wrange
    w_left=i; w_right=i+interval;
    gs_right = GS(w_right, y, sigma, a, f, beta);
    S = S + interval/2 * (gs_left + gs_right);
    gs_left = gs_right;
    gs = [gs, gs_right];
end
%plot(wrange,gs(1:end-1),'.'); pause;
%% To get a feeling on whether numerical integration is working fine, 
%% uncomment the line above, set K=1 at the top, and test with large f. 
%% The curves should always look well covered by points.
end

% GaussStudent function value
function [gs] = GS(w, y, sigma, a, f, beta)
    gs = 0;
    if sigma
        gs = sqrt(1/(2*pi))/sigma * exp(-(y-w)*(y-w)/2/sigma^2) * ...
             sqrt(f) * (1 + w*w*f/2)^(-a-.5) * gamma(a+.5)/gamma(a) * ...
             sqrt(1/(2*pi)) * f^beta;
    end    
end


function [f,zeta] = variational(y, sigma, a, tol,beta)

sigmasq = sigma^2;
K = length(y);

zeta = ones(size(y));
f   = 1;

% auxiliary variables
S = ones(size(y));
v = ones(size(y));

epsilon = Inf;
Obj = Inf;

while epsilon > tol
    for k = 1:K
        S(k) = sigmasq * zeta(k) /(sigmasq*f + zeta(k));
        v(k) = zeta(k) *    y(k) /(sigmasq*f + zeta(k));
        zeta(k)  = (1 + .5*f*(v(k)^2+S(k)))/(a+.5);
    end
    f = K*(.5+beta)/(.5*sum( (v.^2 + S)./zeta ));
    obj = sum(.5* (y-v).^2/sigmasq + .5* (v.^2 + S)*f./zeta - .5 * log(S) ...
              +.5* log(sigmasq) + .5* S/sigmasq - (.5+beta) * log(f)  ...
              + 1./zeta +(a+.5) * log(zeta));
    epsilon = Obj(end)-obj;
    Obj = [Obj,obj];
end
end


function [f,sigmasq,zeta] = variational_sigmaunknown(y, a, tol, beta)

K = length(y);

zeta = ones(size(y));
f   = 1;
sigmasq = 1;

% auxiliary variables
S = ones(size(y));
v = ones(size(y));

epsilon = Inf;
Obj = Inf;

while epsilon > tol
    for k = 1:K
        S(k) = sigmasq * zeta(k) /(sigmasq*f + zeta(k));
        v(k) = zeta(k)  *   y(k) /(sigmasq*f + zeta(k));
        zeta(k)  = (1 + .5*f*(v(k)^2+S(k)))/(a+.5);
    end
    f = K*(.5+beta)/(.5*sum( (v.^2 + S)./zeta ));
    sigmasq = mean((y-v).^2+S);
    obj = sum(.5* (y-v).^2/sigmasq + .5* (v.^2 + S)*f./zeta - .5 * log(S) ...
              +.5* log(sigmasq) + .5* S/sigmasq - (.5+beta) * log(f)  ...
              + 1./zeta +(a+.5) * log(zeta));
    epsilon = Obj(end)-obj;
    Obj = [Obj,obj];
end

end
