function [f, w, bound] = variational_lasso(y, sigma, groups, a, beta, numiter, objhowoften, tol)
% Estimate f and w based on data. Here f is restricted to have the same value for
% any group (hence the similarity to the LASSO, as the LASSO has an uniform prior
% for each variable).
%
% Input: y -- a P x K matrix. For each k=1...K, y(:,k) ~ N(w(:,k), sigma^2 I).
%        sigma -- a scalar: the square root of the variance of the Gaussian noise
%        groups --  an NG x P binary matrix of the considered groups: for i=1...NG,
%                   the i-th row encodes which of the P variables make up the
%                   i-th group
%        a -- a scalar: the a parameter of the Student distribution
%        beta -- a scalar: the regularization parameter
%        numiter -- the number of iterations to be performed
%        objhowoften -- every how many iterations the objective should be computed
%        tol -- the minimal tolerated difference between two consecutively
%                     computed values of the objective (also determining the
%                     number of iterations)
% Output: f -- an NG x 1 vector with all entries equal -- an estimate of the relevance for each group
%         w -- a P x K matrix of estimates of the true w
%         bound -- a scalar: the value of the upper bound on the negative
%                  log-likelihood
%
% Copyright 2014 Nino Shervashidze    

sigmasq = sigma^2;
P = size(y,1);
K = size(y,2);

[NG, cards, inds] = get_datastructures(groups);

if nargin < 7 || objhowoften > numiter objhowoften = numiter; end
if nargin < 8 tol = 0; end

% initialize f
f = ones(NG, 1);

% auxiliary variables
zeta = ones(NG, K);
trS = ones(NG, K);
vnormsq = ones(NG, K);

Obj = [Inf];
log_det_term = zeros(1,K);
trace_term = zeros(1,K);
epsilon = 1;
c=1;
while c <= numiter && epsilon > tol
    % update zeta and f
    f_denom = zeros(NG,1);
    for k=1:K
        zeta(:,k)    = (1 + .5*f.*(vnormsq(:,k) + trS(:,k)) ) ./ (a + cards/2);
        for g=1:NG
            f_denom(g) = f_denom(g) + (vnormsq(g,k) + trS(g,k))/zeta(g,k);
        end
    end
    fvalue = K*sum(cards+2*beta)/sum(f_denom);
    f = repmat(fvalue,size(f));
    
    % update trS and vnormsq
    for k = 1:K
        kappa = zeta(:,k)./f;
        kappasq = kappa.^2;
        xi = groups'*kappa;
        for g=1:NG
            trS(g,k)     = cards(g)*kappa(g) - kappasq(g)*sum(1./xi(inds{g})) +...
                sigmasq*kappasq(g)*sum(1./(xi(inds{g}).*(xi(inds{g})+sigmasq)));
            vnormsq(g,k) = kappasq(g)*sum((y(inds{g},k)./(xi(inds{g})+sigmasq)).^2);
        end
    end

    if rem(c,objhowoften)==0
        % compute w and auxiliary terms for the objective
        w = zeros(P,K);
        for k = 1:K
            kappa = zeta(:,k)./f;
            kappasq = kappa.^2;
            xi = groups'*kappa;
            for g=1:NG
                v = zeros(P,1);
                v(inds{g}) = kappa(g)*(y(inds{g},k)./(xi(inds{g})+sigmasq));
                w(:,k) = w(:,k) + v;
            end
            log_det_term(k) = .5 * sum(log(sigmasq+xi)) + .5 * cards'*log(f./zeta(:,k));
            trace_term(k) = .5 * sum(xi./(xi+sigmasq));
        end
        % compute the objective
        [obj] = compute_obj(y, w, sigmasq, a, beta, f, zeta, trS, vnormsq, log_det_term, trace_term, cards);
        disp(['iter ', num2str(c), ' obj ', num2str(obj), ' f ', num2str(f') ]);
        epsilon = Obj(end)-obj;
        Obj = [Obj,obj];
        bound = obj + K*sum(beta.*log(f));
    end
    c = c+1;
end

end


function [obj] = compute_obj(y, w, sigmasq, a, beta, f, zeta, trS, vnormsq, log_det_term, trace_term, cards)
P = size(y,1);
K = size(y,2);
NG = size(f,1);

obj = 0;
for k=1:K
    % compute the objective for k
    obj = obj +.5/sigmasq * (y(:,k) - w(:,k))'*(y(:,k) - w(:,k)) ...
          + .5 * sum(f./zeta(:,k) .* (vnormsq(:,k) + trS(:,k))) ...
          + log_det_term(k) ...
          + P/2 * log(2*pi) ...
          + trace_term(k) ...
          - .5 * sum(cards) ...
          + sum(1./zeta(:,k)) + (a+cards/2)' * log(zeta(:,k)) ...
          - sum(a+cards/2) + (a+cards/2)' * log(a+cards/2) ...
          - sum(log(gamma(a+cards/2))) + NG*log(gamma(a)) ...
          - (beta+cards/2)' * log(f);
end

end
